# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-09-20 06:45
from __future__ import unicode_literals

from decimal import Decimal
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('experts', '0006_expertrequest_issue'),
    ]

    operations = [
        migrations.AddField(
            model_name='expertrequest',
            name='article_code',
            field=models.CharField(blank=True, max_length=30),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='common_tos',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='contract_methodology',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='milestones',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='objectives',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='out_of_scope',
            field=models.TextField(blank=True, help_text='Elements that are explicitly excluded from the contract'),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='price',
            field=models.DecimalField(decimal_places=7, default=0, max_digits=22, validators=[django.core.validators.MinValueValidator(Decimal('0'))]),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='product_code',
            field=models.CharField(blank=True, max_length=30),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='recurring_billing',
            field=models.BooleanField(default=False, help_text='Defines whether expert request has to be billed every month or only once'),
        ),
        migrations.AlterField(
            model_name='expertcontract',
            name='team',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='structure.Project'),
        ),
    ]
